# Generated by Django 5.0.3 on 2024-04-11 08:24

import colorfield.fields
import django_better_admin_arrayfield.models.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("color", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="colorgradient",
            name="colors",
            field=django_better_admin_arrayfield.models.fields.ArrayField(
                base_field=colorfield.fields.ColorField(
                    default="#FF0000", image_field=None, max_length=25, samples=None
                ),
                default=list,
                size=None,
            ),
        ),
    ]
