# Generated by Django 5.0.6 on 2024-08-20 12:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("documents", "0004_auto_20240103_0958"),
    ]

    operations = [
        migrations.AddField(
            model_name="document",
            name="valid_from",
            field=models.DateField(blank=True, null=True, verbose_name="Valid From"),
        ),
        migrations.AddField(
            model_name="document",
            name="valid_until",
            field=models.DateField(blank=True, null=True, verbose_name="Valid Until"),
        ),
        migrations.AddField(
            model_name="documenttype",
            name="system_key",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name="documentmodelrelationship",
            name="content_object_repr",
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
    ]
