# Generated by Django 5.0.9 on 2024-11-11 12:00

import colorfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("example_app", "0006_league_season_period_player_is_active_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="player",
            options={"ordering": ("order",), "verbose_name": "Player", "verbose_name_plural": "Players"},
        ),
        migrations.AlterModelOptions(
            name="team",
            options={"ordering": ("order",), "verbose_name": "Team", "verbose_name_plural": "Teams"},
        ),
        migrations.AddField(
            model_name="eventtype",
            name="color",
            field=colorfield.fields.ColorField(
                blank=True, default=None, image_field=None, max_length=25, null=True, samples=None
            ),
        ),
        migrations.AddField(
            model_name="player",
            name="order",
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False, verbose_name="order"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="team",
            name="order",
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False, verbose_name="order"),
            preserve_default=False,
        ),
    ]
