{% extends "email_base_template.html" %}
{% load i18n %}
{% block content %}
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <p>
            <b>{% translate "Reset Password" %}</b>
        </p>
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <p>{% blocktranslate %}You're receiving this email because you requested a password reset for your user account at {{ site_name }}.{% endblocktranslate %}</p>
        <p>{% translate "Please click and choose a new password" %}</p>
    </td>

    <td style="background-color: #eaf2ff;"></td>
</tr>
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <table cellspacing="0" cellpadding="0">
            <tr>
                <td style="border-radius: 10px;" bgcolor="#f76f5b">
                    <a class="link" href="{{ protocol }}://{{ domain }}{% url 'wbcore:authentication:password_reset_confirm' uidb64=uid token=token %}" target="_blank" style="padding: 8px 12px; border: 1px solid #ED2939; border-radius: 10px;font-family: Helvetica, Arial, sans-serif;font-size: 14px; color: #ffffff;text-decoration: none;font-weight:bold;display: inline-block;">
                        {% translate "Reset Password" %}
                    </a>
                </td>
            </tr>
        </table>
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
<tr>
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
{% endblock content %}
