{% extends 'base.html' %}
{% load i18n %}
{% block content%}
<div class="container-500">
  <h1>{% translate "Password Reset" %}</h1>
</div>
<h1>{% translate "Password reset sent" %}</h1>
<p>{% translate "We’ve emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly." %}</p>
<p style="width: 60%; margin: 0 auto ">{% translate "If you don’t receive an email, please make sure you’ve entered the address you registered with, and check your spam folder." %}</p>

{% endblock%}
