# Generated by Django 5.0.4 on 2024-05-06 08:57

import picklefield.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("io", "0005_exportsource_data_alter_exportsource_query_str_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="exportsource",
            name="query_params",
            field=picklefield.fields.PickledObjectField(
                blank=True, editable=False, verbose_name="SQL query parameters to be used with the sql query"
            ),
        ),
    ]
