# Generated by Django 4.1 on 2023-03-30 09:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0001_initial"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="notificationusertoken",
            constraint=models.UniqueConstraint(
                fields=("user", "token", "device_type"), name="unique_user_token_device"
            ),
        ),
    ]
