# Generated by Django 5.0.4 on 2024-05-07 07:25

import picklefield.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("io", "0006_alter_exportsource_query_params"),
    ]

    operations = [
        migrations.AlterField(
            model_name="exportsource",
            name="query_params",
            field=picklefield.fields.PickledObjectField(
                blank=True,
                default=list,
                editable=False,
                verbose_name="SQL query parameters to be used with the sql query",
            ),
        ),
    ]
