# Generated by Django 4.2.6 on 2024-01-31 10:40

import wbcore.contrib.icons.models
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("agenda", "0008_alter_calendaritem_item_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="calendaritem",
            name="icon",
            field=wbcore.contrib.icons.models.IconField(blank=True, max_length=128, null=True, verbose_name="Icon"),
        ),
    ]
