# Generated by Django 4.2.9 on 2024-01-26 15:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("workflow", "0003_alter_condition_attribute_name_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userstep",
            name="assignee_method",
            field=models.CharField(
                blank=True,
                choices=[
                    ("manager_of_instance_assignee", "Instance Assignee's Manager"),
                    ("weighted_random", "Weighted Random Group Member"),
                    ("random_group_member", "Random Group Member"),
                ],
                max_length=64,
                null=True,
                verbose_name="Assignee Method",
            ),
        ),
    ]
