# Generated by Django 4.1.8 on 2023-04-28 13:36

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcore", "0001_initial_squashed_squashed_0010_preset_appliedpreset"),
    ]

    operations = [
        migrations.CreateModel(
            name="GenericModel",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                ("label", models.CharField(max_length=256)),
            ],
            options={
                "managed": False,
            },
        ),
    ]
