# Generated by Django 5.0.6 on 2024-06-19 06:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0008_bankingcontact_access"),
    ]

    operations = [
        migrations.RenameField(
            model_name="entry",
            old_name="external_identfier",
            new_name="external_identifier",
        ),
        migrations.AlterField(
            model_name="entry",
            name="external_identifier",
            field=models.CharField(blank=True, max_length=128, null=True, unique=True),
        ),
    ]
