# Generated by Django 4.1.8 on 2023-04-14 14:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("agenda", "0002_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="calendaritem",
            name="endpoint_basename",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="calendaritem",
            name="item_type",
            field=models.CharField(
                choices=[("AbsenceRequest", "Absence Request"), ("Activity", "Activity"), ("Task", "Task")],
                default="Activity",
                max_length=255,
                verbose_name="Type",
            ),
        ),
        migrations.AlterField(
            model_name="calendaritem",
            name="item_type",
            field=models.CharField(
                choices=[
                    ("CalendarItem", "Calendar Item"),
                    ("AbsenceRequest", "Absence Request"),
                    ("Activity", "Activity"),
                    ("Task", "Task"),
                ],
                default="CalendarItem",
                max_length=255,
                verbose_name="Type",
            ),
        ),
    ]
