# Generated by Django 4.2.7 on 2023-12-06 13:22

from django.db import migrations


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ("authentication", "0005_user_external_calendar_settings"),
    ]

    operations = []
