# Generated by Django 4.2.6 on 2023-10-24 14:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0004_entry_is_draft_entry"),
    ]

    operations = [
        migrations.AddField(
            model_name="entry",
            name="salutation",
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name="Salutation"),
        ),
    ]
