from .endpoints import (
    PlayerTeamEndpointConfig,
    MatchStadiumEndpointConfig,
    EventMatchEndpointConfig,
    TeamStadiumEndpointConfig,
    EventTypeSportEndpointConfig,
    EventEndpointConfig,
    MatchLeagueEndpointConfig,
    LeagueSportEndpointConfig,
    TeamResultsEndpointConfig,
    LeagueStatisticsEndpointConfig,
)
