# Generated by Django 5.0.6 on 2024-06-05 14:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0007_alter_bankingcontact_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="bankingcontact",
            name="access",
            field=models.ManyToManyField(blank=True, related_name="access_to_bank_accounts", to="directory.person"),
        ),
    ]
