# Generated by Django 4.1.8 on 2023-05-23 08:13

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcore", "0011_genericmodel"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Notification",
        ),
    ]
